#version 330

//subroutine void RenderPassType();
//subroutine uniform RenderPassType RenderPass;

layout (location = 0) in vec3 VertexPosition;
layout (location = 1) in vec3 VertexVelocity;
layout (location = 2) in float VertexStartTime;
layout (location = 3) in vec3 VertexInitialVelocity;
layout (location = 4) in float StartState;


out vec3 Position;   // To transform feedback
out vec3 Velocity;   // To transform feedback
out float StartTime; // To transform feedback
out float StateOut;

out float Transp;    // To fragment shader


uniform float Time;  // Simulation time
uniform float H;     // Elapsed time between frames
uniform vec3 Accel;  // Particle acceleration
uniform float ParticleLifetime;  // Particle lifespan
uniform float MinParticleSize = 10.0;
uniform float MaxParticleSize = 200.0;

uniform vec3 movepos;
uniform bool renderP;

uniform mat4 MVP;

//subroutine (RenderPassType)
void update() {

    // Update position & velocity for next frame
    Position = VertexPosition;
    Velocity = VertexVelocity;
    StartTime = VertexStartTime;
	StateOut = StartState;

    if( Time >= StartTime ) {

        float age = Time - StartTime;
		if (StartState == 0.0) {
			// start
			Position = movepos;//vec3(0.0,0.0,0.0);
			StateOut = 1.0;
		} 

        if( age > ParticleLifetime ) {
            // The particle is past it's lifetime, recycle.
			// new pos
       //     Position = movepos;//vec3(1.0,0.0,0.0);
            Velocity = VertexInitialVelocity;
            StartTime = Time;
			StateOut = 0.0;
        } else {
            // The particle is alive, update.
            Position += Velocity * H;
            Velocity += Accel * H;
        }
    }
}

//subroutine (RenderPassType)
void render() {
    float age = Time - VertexStartTime;
    Transp = 0.0;
    if( Time >= VertexStartTime ) {
        float agePct = age/ParticleLifetime;
        Transp = 1.0 - agePct;
		//if (Transp < 0.0 ) Transp = 0.0;
        gl_PointSize = mix(MinParticleSize,MaxParticleSize,agePct);
    }
	
	//gl_PointSize = 1000;
	//gl_Position = MVP * vec4(0,0,0,1.0);
	
    gl_Position = MVP * vec4(VertexPosition, 1.0);
}

void main()
{
    // This will call either render() or update()
   // RenderPass();
  // bool test = true;
   if (renderP) {
	render();
	  } else {
	   update();
  }
}
