#version 330 core
#extension GL_EXT_texture_array : enable

uniform sampler2D gSampler[1];

smooth in vec2 vTexCoord;
smooth in vec3 vNormal;
smooth in vec3 vWorldPos;
smooth in vec4 vEyeSpacePos;

in vec4 ShadowCoord;
in vec4 vPos;

smooth in float islf;


uniform sampler2D lensFlare;
uniform sampler2D shadowMap;
uniform sampler2D lensColor;
uniform sampler2D lensStar;
uniform sampler2DArray cascShadowMap;
uniform mat4 depthBiasVP[4];
uniform mat4 depthNormals;
uniform bool renderCascades;

uniform vec4 fard;

out vec4 color;

// sample offsets
const int nsamples = 8;
uniform vec4 offset[nsamples] = { vec4(0.000000, 0.000000, 0.0, 0.0),
								  vec4(0.079821, 0.165750, 0.0, 0.0),
								  vec4(-0.331500, 0.159642, 0.0, 0.0),
								  vec4(-0.239463, -0.497250, 0.0, 0.0),
								  vec4(0.662999, -0.319284, 0.0, 0.0),
								  vec4(0.399104, 0.828749, 0.0, 0.0),
								  vec4(-0.994499, 0.478925, 0.0, 0.0),
								  vec4(-0.558746, -1.160249, 0.0, 0.0) };

in VS_OUT {
vec4 color;
} fs_in;

struct DirectionalLight
{
	vec3 vColor;
	vec3 vDirection;

	float fAmbient;
};
uniform DirectionalLight sunLight;
DirectionalLight test;

vec4 GetDirectionalLightColor(DirectionalLight dirLight, vec3 vNormal)
{
	float fDiffuseIntensity = max(0.0, dot(vNormal, normalize(-dirLight.vDirection)));
	float fMult = clamp(dirLight.fAmbient+fDiffuseIntensity, 0.0, 1.0);
	return vec4(dirLight.vColor*fMult, 1.0);
}

float getCoef(vec4 shadowCoord){
	
	float casc = texture2DArray(cascShadowMap, shadowCoord.xyz).x;
	float diff = (casc-shadowCoord.w);
	if(casc==1.0f) diff = 1.0f;
	//return clamp( diff*250.0+1.0, 0.0, 1.0);
	//return clamp( 1-diff*250, 0.0, 1.0);
	
	return diff;
	//return casc;
	
}

int gloIndex;

float shadowCoef(){
	const float scale = 2.0/4096.0;
	int index = 0;
	if(gl_FragCoord.z < fard.x)
		index = 0;
	else if(gl_FragCoord.z < fard.y)
		index = 1;
	else if(gl_FragCoord.z < fard.z)
		index = 2;
	else if(gl_FragCoord.z<fard.w)
		index=3;
	

	gloIndex=index;
	vec4 shadow_coord = depthBiasVP[index]*vec4(vWorldPos,1.0);
	
	
	vec4 light_normal4 = depthNormals[index]*vec4(vNormal, 0.0);
	vec3 light_normal = normalize(light_normal4.xyz);

	float d = -dot(light_normal, shadow_coord.xyz);

	//shadow_coord.xy/=shadow_coord.w;

	shadow_coord.w=shadow_coord.z;
	shadow_coord.z=float(index);
	float shadow_coef = getCoef(shadow_coord);

	//shadow_coef = texture2DArray(cascShadowMap, vec3(ShadowCoord.st,1.0)).x;
	//shadow_coef = texture2DArray(cascShadowMap, vec3(shadow_coord.st,shadow_coord.z)).x;
	
	for(int i=1;i<nsamples;i++){
		vec4 shadow_lookup = shadow_coord+scale*offset[i];
		float lookup_z = -(light_normal.x*shadow_lookup.x + light_normal.y*shadow_lookup.y + d)/light_normal.z;
		//shadow_lookup.w = lookup_z;
		shadow_coef += getCoef(shadow_lookup);
	}

	shadow_coef /= nsamples;


	return shadow_coef;
	//return getCoef(shadow_coord);
	

}

void main()
{

	vec3 vNormalized = normalize(vNormal);
	
	vec4 vTexColor = vec4(0.0);

	float fScale = vWorldPos.y/1.0;

	vTexColor = texture2D(gSampler[0], vTexCoord);

	

	vec4 vFinalTexColor = vTexColor;
	vec4 vDirLightColor = GetDirectionalLightColor(sunLight, vNormal);

	test.vColor=vec3(1.0,1.0,1.0);
	test.vDirection=vec3(10,-10,0);
	test.fAmbient=0.25;

	vDirLightColor = GetDirectionalLightColor(test,vNormal);
	
	float lf_flag=islf;
	
	//color = vec4(1.0,1.0,1.0,1.0);
	float bias = -0.7;
	float scale = 7.0;
	vec2 texcoord = vTexCoord;

	int ughosts = 5;
	float dispersal = 0.8;
	
	vec2 texelSize = 1.0/vec2(textureSize(lensFlare,0));

	vec2 ghostVec = (vec2(0.5)-texcoord)*dispersal;
	vec4 result = vec4(0.0);
	for(int i=0;i<ughosts;++i){
		vec2 offset = fract(texcoord+ghostVec*float(i));

		float weight = length(vec2(0.5)-offset) / length(vec2(0.5));
		weight= pow(1.0-weight, 1.0);
		weight*=1.0;
		result+=texture(lensFlare,offset)*weight;
	}


	//vec4 lf = max(vec4(0.0), texture(lensFlare,texcoord)+bias) * scale;
	
	vec4 lf=(result);
	//lf=texture(lensFlare,vTexCoord);
	lf+=texture(lensStar, vTexCoord.xy);
	lf.a=0.3;
	if(lf.x<0.8 || lf.y<0.8 || lf.z<0.8) lf=vec4(0.0);
	//else lf=vec4(0.0);

	lf*=texture(lensColor, vTexCoord);
	//lf=texture(lensColor, vTexCoord);
	//lf=texture(lensFlare,vTexCoord);

	vec2 haloVec = normalize(ghostVec)*0.1;
	float weight = length(vec2(0.5)-fract(vTexCoord+haloVec)) / length(vec2(0.5));
	weight = pow(1.0-weight,5.0);

	if(lf.x>0.1 || lf.y>0.1 || lf.z>0.1 || false)
	lf += texture(lensFlare, vTexCoord+haloVec)*weight;
	
	vec4 scwd = ShadowCoord/ShadowCoord.w;
	
	vec4 visibility = texture(shadowMap, scwd.st);
	
	float cascV = shadowCoef();
	

	//cascV = texture2DArray(cascShadowMap, vec3(scwd.st,1.0)).x;
	//if(cascV<1.0)cascV=0.3;

	vec4 cascColor;
	if(gloIndex==0) cascColor = vec4(1,1,0,1);
	if(gloIndex==1) cascColor = vec4(0,1,0,1);
	if(gloIndex==2) cascColor = vec4(0,1,1,1);
	if(gloIndex==3) cascColor = vec4(1,0,1,1);
	
	if(lf_flag>0.0){
		color = vFinalTexColor*vDirLightColor;//fs_in.color;
		//color *= visibility;
		color *= cascV;
		if(renderCascades) color *= cascColor; // visualization of cascades
		
	}
	else color=lf;
	
	//color = vFinalTexColor*vDirLightColor;//fs_in.color;
}

